//-------------------------------------------------------------------------------------------------------
// VST Plug-Ins SDK
// Version 2.4		$Date: 2006/11/13 09:08:27 $
//
// Category     : VST 2.x SDK Samples
// Filename     : SSInst.cpp
// Created by   : Steinberg Media Technologies
// Description  : 2 oscilattor synthesizer

//  2006, Steinberg Media Technologies, All Rights Reserved
//-------------------------------------------------------------------------------------------------------

#include "SSInst.h"
#include "gmnames.h"

//-------------------------------------------------------------------------------------------------------
AudioEffect* createEffectInstance (audioMasterCallback audioMaster)
{
	return new SSInst (audioMaster);
}

//-----------------------------------------------------------------------------------------
// SSInstProgram
//-----------------------------------------------------------------------------------------
SSInstProgram::SSInstProgram ()
{
	// Default Program Values
	fWaveA = 0.f;	// Saw
	fFreqA     =.0f;
	fVolA   = .2f;

	fWaveB = 1.f;	// Pulse
	fFreqB     = .05f;	// slightly higher
	fVolB   = .2f;
	
	fWaveC = 0.f; // Sine
	fFreqC = .05f;
	fVolC = .2f;
	
	fVolume    = .9f;
	vst_strncpy (name, "Def", kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
// SSInst
//-----------------------------------------------------------------------------------------
SSInst::SSInst (audioMasterCallback audioMaster)
: AudioEffectX (audioMaster, kNumPrograms, kNumParams)
{
	// initialize programs
	programs = new SSInstProgram[kNumPrograms];
	for (VstInt32 i = 0; i < 16; i++)
		channelPrograms[i] = i;

	if (programs)
		setProgram (0);
	
	if (audioMaster)
	{
		setNumInputs (0);				// no inputs
		setNumOutputs (kNumOutputs);	// 2 outputs, 1 for each oscillator
		canProcessReplacing ();
		isSynth ();
		setUniqueID ('SSInst');			
	}

	initProcess ();
	suspend ();
}

//-----------------------------------------------------------------------------------------
SSInst::~SSInst ()
{
	if (programs)
		delete[] programs;
}

//-----------------------------------------------------------------------------------------
void SSInst::setProgram (VstInt32 program)
{
	if (program < 0 || program >= kNumPrograms)
		return;
	
	SSInstProgram *ap = &programs[program];
	curProgram = program;
	
	fWaveA     = ap->fWaveA;
	fFreqA     = ap->fFreqA;
	fVolA      = ap->fVolA;

	fWaveB     = ap->fWaveB;
	fFreqB     = ap->fFreqB;
	fVolB      = ap->fVolB;
	
	fVolume    = ap->fVolume;
}

//-----------------------------------------------------------------------------------------
void SSInst::setProgramName (char* name)
{
	vst_strncpy (programs[curProgram].name, name, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
void SSInst::getProgramName (char* name)
{
	vst_strncpy (name, programs[curProgram].name, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
void SSInst::getParameterLabel (VstInt32 index, char* label)
{
	switch (index)
	{
		case kWaveA:
		case kWaveB:
			vst_strncpy (label, "Waveform", kVstMaxParamStrLen);
			break;

		case kFreqA:
		case kFreqB:
			vst_strncpy (label, "Hz", kVstMaxParamStrLen);
			break;

		case kVolA:
		case kVolB:
		case kVolume:
			vst_strncpy (label, "dB", kVstMaxParamStrLen);
			break;
	}
}

//-----------------------------------------------------------------------------------------
void SSInst::getParameterDisplay (VstInt32 index, char* text)
{
	text[0] = 0;
	switch (index)
	{
		case kWaveA:
			if (fWaveA < .33f)
				vst_strncpy (text, "Sawtooth", kVstMaxParamStrLen);
			else if (fWaveA < .66f)
				vst_strncpy (text, "Pulse", kVstMaxParamStrLen);
			else if (fWaveA < 1.0f)
				vst_strncpy (text, "Sine", kVstMaxParamStrLen) ;
			break;

		case kFreqA:		float2string (fFreqA, text, kVstMaxParamStrLen);	break;
		case kVolA:		dB2string (fVolA, text, kVstMaxParamStrLen);	break;
		
	case kWaveB:
			if (fWaveB < .33f)
				vst_strncpy (text, "Sawtooth", kVstMaxParamStrLen);
			else if (fWaveB < .66f)
				vst_strncpy (text, "Pulse", kVstMaxParamStrLen);
			else if (fWaveB < 1.0f)
				vst_strncpy (text, "Sine", kVstMaxParamStrLen);
			break;
			
		case kFreqB:	float2string (fFreqB, text, kVstMaxParamStrLen);	break;
		case kVolB:		dB2string (fVolB, text, kVstMaxParamStrLen);	break;
		case kVolume:		dB2string (fVolume, text, kVstMaxParamStrLen);	break;
	}
}

//-----------------------------------------------------------------------------------------
void SSInst::getParameterName (VstInt32 index, char* label)
{
	switch (index)
	{
		case kWaveA:	vst_strncpy (label, "Waveform A", kVstMaxParamStrLen);	break;
		case kFreqA:	vst_strncpy (label, "Freq. A", kVstMaxParamStrLen);	break;
		case kVolA:		vst_strncpy (label, "Vol. A", kVstMaxParamStrLen);	break;
		case kWaveB:	vst_strncpy (label, "Waveform B", kVstMaxParamStrLen);	break;
		case kFreqB:		vst_strncpy (label, "Freq. B", kVstMaxParamStrLen);	break;
		case kVolB:		vst_strncpy (label, "Vol. B", kVstMaxParamStrLen);	break;
		case kVolume:		vst_strncpy (label, "Volume", kVstMaxParamStrLen);	break;
	}
}

//-----------------------------------------------------------------------------------------
void SSInst::setParameter (VstInt32 index, float value)
{
	SSInstProgram *ap = &programs[curProgram]; 
	switch (index)
	{
		case kWaveA:	fWaveA	= ap->fWaveA	= value;	break;
		case kFreqA:	fFreqA 	= ap->fFreqA	= value;	break;
		case kVolA:		fVolA	= ap->fVolA		= value;	break;
		case kWaveB:	fWaveB	= ap->fWaveB	= value;	break;
		case kFreqB:	fFreqB 	= ap->fFreqB	= value;	break;
		case kVolB:		fVolB	= ap->fVolB		= value;	break;
		case kVolume:	fVolume	= ap->fVolume	= value;	break;
	}
}

//-----------------------------------------------------------------------------------------
float SSInst::getParameter (VstInt32 index)
{
	float value = 0;
	switch (index)
	{
		case kWaveform1:	value = fWaveform1;	break;
		case kFreq1:		value = fFreq1;		break;
		case kVolume1:		value = fVolume1;	break;
		case kWaveform2:	value = fWaveform2;	break;
		case kFreq2:		value = fFreq2;		break;
		case kVolume2:		value = fVolume2;	break;
		case kVolume:		value = fVolume;	break;
	}
	return value;
}

//-----------------------------------------------------------------------------------------
bool SSInst::getOutputProperties (VstInt32 index, VstPinProperties* properties)
{
	if (index < kNumOutputs)
	{
		vst_strncpy (properties->label, "SSInst ", 63);
		char temp[11] = {0};
		int2string (index + 1, temp, 10);
		vst_strncat (properties->label, temp, 63);

		properties->flags = kVstPinIsActive;
		if (index < 2)
			properties->flags |= kVstPinIsStereo;	// make channel 1+2 stereo
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------------
bool SSInst::getProgramNameIndexed (VstInt32 category, VstInt32 index, char* text)
{
	if (index < kNumPrograms)
	{
		vst_strncpy (text, programs[index].name, kVstMaxProgNameLen);
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------------
bool SSInst::getEffectName (char* name)
{
	vst_strncpy (name, "SSInst", kVstMaxEffectNameLen);
	return true;
}

//-----------------------------------------------------------------------------------------
bool SSInst::getVendorString (char* text)
{
	vst_strncpy (text, "SS Technologies", kVstMaxVendorStrLen);
	return true;
}

//-----------------------------------------------------------------------------------------
bool SSInst::getProductString (char* text)
{
	vst_strncpy (text, "VST Simple Synth", kVstMaxProductStrLen);
	return true;
}

//-----------------------------------------------------------------------------------------
VstInt32 SSInst::getVendorVersion ()
{ 
	return 1000; 
}

//-----------------------------------------------------------------------------------------
VstInt32 SSInst::canDo (char* text)
{
	if (!strcmp (text, "receiveVstEvents"))
		return 1;
	if (!strcmp (text, "receiveVstMidiEvent"))
		return 1;
	return -1;	// explicitly can't do; 0 => don't know
}

//-----------------------------------------------------------------------------------------
VstInt32 SSInst::getNumMidiInputChannels ()
{
	return 1; // we are monophonic
}

//-----------------------------------------------------------------------------------------
VstInt32 SSInst::getNumMidiOutputChannels ()
{
	return 0; // no MIDI output back to Host app
}
